﻿//--------------------------------------------------------------------------------------------------------------------
// Install the Microsoft Speech Platform - Server Runtime (Version 10.2) 
// http://www.microsoft.com/downloads/en/details.aspx?FamilyID=bb0f72cb-b86b-46d1-bf06-665895a313c7&displaylang=en
//
// Install the Microsoft Speech Platform - Software Development Kit (SDK) (Version 10.2)
// http://www.microsoft.com/downloads/en/details.aspx?FamilyID=1b1604d3-4f66-4241-9a21-90a294a5c9a4&displaylang=en
//
// Then install the languages you wish for TTS and SR, among 26 languages
// http://www.microsoft.com/downloads/en/details.aspx?FamilyID=47ffd4e5-e682-4228-8058-dd895252a3c3&displaylang=en
//--------------------------------------------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Media;
using System.Globalization;

// Reference x TTS & SR --> C:\Program Files\Microsoft Speech Platform SDK\Assembly\Microsoft.Speech.dll
using Microsoft.Speech.Recognition;
using Microsoft.Speech.Synthesis;

namespace SpeechSample
{
    public partial class frmSpeechSample : Form
    {
        // TTS stuff
        SpeechSynthesizer _tts = null;
        SoundPlayer _player = new SoundPlayer();

        // SR stuff
        SpeechRecognitionEngine _sre = null;
        Dictionary<string, Array> _dictionary = new Dictionary<string, Array>();

        // Numbers of words in the arrays
        const int _numWords = 5;

        // Define here the words array for your culture and remember to add it to the dictionary in the 'ftmSpeechSample_Load' events
        string[] _words_enUS = new string[_numWords] { "red", "green", "blue", "yellow", "orange" };
        string[] _words_esES = new string[_numWords] { "rojo", "verde", "azul", "amarillo", "naranja" };
        string[] _words_frFR = new string[_numWords] { "rouge", "vert", "bleu", "jaune", "orange" };
        string[] _words_itIT = new string[_numWords] { "rosso", "verde", "blu", "giallo", "arancione" };

        // Define an array of objects with Colors to match with the index of the corresponding word in the array
        object[] _colors = new object[_numWords] { Color.Red, Color.Green, Color.Blue, Color.Yellow, Color.Orange };        

        // Culture of the SR
        string _strCulture = "";        

        public frmSpeechSample()
        {
            InitializeComponent();
        }

        private void frmSpeechSample_Load(object sender, EventArgs e)
        {
            // Create the TTS object and setup to fire the event 'SpeakCompleted'
            _tts = new SpeechSynthesizer();
            _tts.SpeakCompleted += new EventHandler<SpeakCompletedEventArgs>(tts_SpeakCompleted);

            // Prepare the listView
            listViewTTSVoices.View = View.Details;
            listViewTTSVoices.FullRowSelect = true;
            listViewTTSVoices.HideSelection = false;
            listViewTTSVoices.Columns.Add("Name", 350, HorizontalAlignment.Left);
            listViewTTSVoices.Columns.Add("Culture", 50, HorizontalAlignment.Left);
            listViewTTSVoices.Columns.Add("Gender", 50, HorizontalAlignment.Left);
            listViewTTSVoices.Columns.Add("Age", 50, HorizontalAlignment.Left);
            listViewTTSVoices.Columns.Add("Id", 200, HorizontalAlignment.Left);

            // Enumerate the installed Microsoft Server Speech TTS Voices
            foreach (InstalledVoice voice in _tts.GetInstalledVoices())
            {
                // Prepare a listview row to add
                ListViewItem listItem = new ListViewItem(voice.VoiceInfo.Name);
                listItem.SubItems.Add(voice.VoiceInfo.Culture.ToString());
                listItem.SubItems.Add(voice.VoiceInfo.Gender.ToString());
                listItem.SubItems.Add(voice.VoiceInfo.Age.ToString());
                listItem.SubItems.Add(voice.VoiceInfo.Id.ToString());

                // Add a listview row
                listViewTTSVoices.Items.Add(listItem);
            }

            lblNumInstalledTTSVoices.Text = listViewTTSVoices.Items.Count + " TTS voices found of 26.";
            lblSelectedTTSVoice.Text = "";
            lblSR.Text = "";

            // Setup the dictionary for the different languages 
            _dictionary = new Dictionary<string, Array>();
            _dictionary.Add("en-US", _words_enUS);
            _dictionary.Add("es-ES", _words_esES);
            _dictionary.Add("fr-FR", _words_frFR);
            _dictionary.Add("it-IT", _words_itIT);
            _tts.SpeakAsyncCancelAll();
            _tts.SelectVoice("Microsoft Server Speech Text to Speech Voice (en-US, ZiraPro)");
            SetupSREngine("en-US");
            this.Hide(); 
        }

        // Note: The SpeakCompleted event will finish when TTS stops synthesizing the audio to the stream (which should be really quick), 
        // not when you stop hearing the actual audio (as in the behavior of the  SpeechSynthesizer in System.Speech.Synthesis). 
        // The player itself will handle the playback.
        void tts_SpeakCompleted(object sender, SpeakCompletedEventArgs e)
        {
            _player.Stream.Position = 0;
            _player.Play();
        }

        private void btnTTSSpeech_Click(object sender, EventArgs e)
        {
            Speak(txtTTS.Text);
        }

        public void Speak(string text)
        {
            _player.Stream = new System.IO.MemoryStream();
            _tts.SetOutputToWaveStream(_player.Stream);
            _tts.SpeakAsync(text);
        }

        private void listViewTTSVoices_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Has a Voice been selected from the list ?
            if (listViewTTSVoices.SelectedItems.Count > 0)
            {
                // Yes
                // Change the voice to the selected one
                _tts.SpeakAsyncCancelAll();
                _tts.SelectVoice(listViewTTSVoices.SelectedItems[0].Text);
                
                // Display the selected voice 
                lblSelectedTTSVoice.Text = _tts.Voice.Name;

                btnTTSSpeech.Enabled = true;

                // Get the Culture from the selected TTS Voice
                _strCulture = listViewTTSVoices.SelectedItems[0].SubItems[1].Text;
               
                // Setup also the Speech Recognition Engine for this culture
                SetupSREngine(_strCulture);
                
                panel1.BackColor = this.BackColor;
            }
        }

        private void SetupSREngine(string strCulture)
        {
            listBoxRecognisedWords.Items.Clear();
            listBoxWordsToBeRecognised.Items.Clear();

            // Was the SRE already defined ?
            if (_sre != null)
            {
                // Yes
                _sre = null;
            }

            //-----------------------------------------------------------------------------------------------------
            // For a Culture the files to install are two: one for TTS and one for SR.
            // Example: For en-US the files are: 'MSSpeech_SR_en-US_TELE.msi' and 'MSSpeech_TTS_en-US_Helen.msi'
            // NOTE: If you don't have the file installed you could get an error when try to instance it.
            //-----------------------------------------------------------------------------------------------------
            try
            {
                // Create a new SpeechRecognitionEngine instance.
                _sre = new SpeechRecognitionEngine(new System.Globalization.CultureInfo(strCulture));
            }
            catch (Exception ex)
            {
                lblSR.Text = "";
                toolStripStatusLabel1.Text = "Unable to instance SR engine for culture: " + strCulture + " - " + ex.Message;
                return;
            }

            // Create a simple grammar that recognizes the words
            Choices words = new Choices();

            // See if the dictionary contains the worlds for the selected culture
            if (_dictionary.ContainsKey(strCulture))
            {
                // Yes
                string[] cultureWords = (string[])_dictionary[strCulture];

                // Add the words to be recognised for this culture
                foreach (string s in cultureWords)
                {
                    words.Add(s);
                    listBoxWordsToBeRecognised.Items.Add(s);
                }
            }
            else
            {
                // No
                toolStripStatusLabel1.Text = "Words for this culture are NOT defined yet! Please modify the source code and try again.";
                lblSR.Text = "";
                return;
            }

            GrammarBuilder gb = new GrammarBuilder();
            gb.Culture = new System.Globalization.CultureInfo(strCulture);
            gb.Append(words);

            // Create the actual Grammar instance, and then load it into the speech recognizer.
            Grammar g = new Grammar(gb);
            _sre.LoadGrammar(g);

            // Register a handler for the SpeechRecognized event.
            _sre.SpeechRecognized += new EventHandler<SpeechRecognizedEventArgs>(sre_SpeechRecognized);
            _sre.SetInputToDefaultAudioDevice();
            _sre.RecognizeAsync(RecognizeMode.Multiple);

            // Display the selected culture for the SR
            lblSR.Text = strCulture;
        }

        // Simple handler for the SpeechRecognized event.
        void sre_SpeechRecognized(object sender, SpeechRecognizedEventArgs e)
        {
            // Add to the listbox the recognised speech
            listBoxRecognisedWords.Items.Add(e.Result.Text);

            // See if the dictionary contains the worlds for the selected culture
            if (_dictionary.ContainsKey(_strCulture))
            {
                // Yes
                string[] cultureWords = (string[])_dictionary[_strCulture];

                // Loop through the words defined for this culture
                for (int i = 0; i <= cultureWords.GetUpperBound(0) ; i++)
                {
                    // Is this word be recognised ?
                    if (string.Compare(cultureWords[i], e.Result.Text, true) == 0)
                    { 
                        // Yes
                        // We can take an action and change the color of the panel
                        panel1.BackColor = (Color)_colors[i];
                    }
                }
            }            
        }
    }
}
